
#import <QuartzCore/QuartzCore.h>
#import "GameController.h"
#import "GameBoardView.h"
#import "GameBoardTile.h"
#import "GamePieceView.h"

/*
char	MapDefData[MAP_CELL_COUNT]={
	2,2,2,2,2,2,2,2,2,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,0,0,0,0,0,0,0,0,2,
	2,2,2,2,2,2,2,2,2,2,
};
*/

@implementation GameController

@synthesize	gameView;

////////////////////////////////////////////////////////
//初期化
////////////////////////////////////////////////////////
- (id) initWithBaseView:(UIView*)baseview
{
	self = [super init];
	if (self != nil) {
		
		//ゲームビュー生成
		NSMutableArray*		tileArray = [NSMutableArray array];
		int					x, y, tileIndex = 0;
		for (y=0; y < TILE_H_COUNT; y++) {
			for (x=0; x < TILE_W_COUNT; x++) {
				CGRect	frame = CGRectMake(TILE_POS_X+(x*TILE_WIDTH), 
										   TILE_POS_Y+(y*TILE_HEIGHT), 
										   TILE_WIDTH, TILE_HEIGHT);
				GameBoardTile*	tile = [GameBoardTile gameBoardTileWith:frame 
																 State:0];
				tile.tileIndex = tileIndex;
				tileIndex++;
				[tileArray addObject:tile];
			}
		}
		self.gameView = [[[GameBoardView alloc] initWithFrame:CGRectMake(0, 0, 
																		 baseview.frame.size.width, 
																		 baseview.frame.size.height) 
															  Tiles:tileArray 
															BGImage:[UIImage imageNamed:@"bg00.png"]] autorelease];
		[baseview addSubview:self.gameView];
		self.gameView.delegate = self;
		
		//キャラクターを生成
		//うさぎ
		{
			NSMutableArray*	array = [NSMutableArray array];
			int			i;
			for(i=0; i < 12; i++){
				NSString*	name = [NSString stringWithFormat:@"rabit00_%03d.png", i];
				UIImage*	image = [UIImage imageNamed:name];
				[array addObject:image];
			}
			pieceView = [[GamePieceView alloc] initWithFrame:CGRectMake(0, 0, 48, 48) 
													  Images:array 
													Delegate:self];
			pieceView.type = TYPE_RABBIT;
			//配置
			[gameView addSubview:pieceView];
			GameBoardTile*	tile = [self.gameView tileAtIndex:44];
			tile.piece = pieceView;
			pieceView.tile = tile;
			pieceView.center = [tile center];
			//タイマースタート
			[pieceView timerStart];
		}
		
		//オオカミ
		{
			NSMutableArray*	array = [NSMutableArray array];
			int			i;
			for(i=0; i < 12; i++){
				NSString*	name = [NSString stringWithFormat:@"wlof00_%03d.png", i];
				UIImage*	image = [UIImage imageNamed:name];
				[array addObject:image];
			}
			enemyView = [[GamePieceView alloc] initWithFrame:CGRectMake(0, 0, 48, 48) 
													  Images:array 
													Delegate:self];
			enemyView.type = TYPE_WLOF;
			enemyView.speed = 30;
			//配置
			[gameView addSubview:enemyView];
			GameBoardTile*	tile = [self.gameView tileAtIndex:22];
			tile.piece = enemyView;
			enemyView.tile = tile;
			enemyView.center = [tile center];
			//タイマースタート
			[enemyView timerStart];
		}
		
		
		
	}
	
	return	self;
}

////////////////////////////////////////////////////////
//解放
////////////////////////////////////////////////////////
- (void)dealloc {
	[pieceView release];
	[gameView release];
    [super dealloc];
}


////////////////////////////////////////////////////////
//近くにいるかチェック
////////////////////////////////////////////////////////
-(BOOL)isNearPlayerPiece:(GamePieceView*)piece {
	
	BOOL	ret = NO;
	//オオカミの場合...
	if(piece.type==TYPE_WLOF) {
		int				tileIndex, targetIndex;
		//オオカミの現在の位置
		tileIndex = piece.tile.tileIndex;
		//うさぎの現在の位置
		targetIndex = pieceView.tile.tileIndex;
		if( tileIndex==targetIndex || 
		   tileIndex+1==targetIndex || 
		   tileIndex-1==targetIndex || 
		   tileIndex+TILE_W_COUNT==targetIndex ||
		   tileIndex-TILE_W_COUNT==targetIndex ) {
			ret = YES;
		}
		return	ret;
	}
	else {
		return	NO;
	}
}


#pragma mark -
#pragma mark -- GameBoardViewDelegate --

////////////////////////////////////////////////////////
//タッチアップ
////////////////////////////////////////////////////////
-(void) gameBoardViewTouchUp:(GameBoardView*)gmView Location:(CGPoint)touchPt Taps:(int)taps Event:(UIEvent*)event
{
	GameBoardTile*	tile = [gmView tileAtPoint:touchPt];
	if( tile ){
		[pieceView moveWith:tile];
	}

}


#pragma mark -
#pragma mark -- GamePieceViewDelegate --

////////////////////////////////////////////////////////
//スタート
////////////////////////////////////////////////////////
-(void) gamePieceCAMoveStart:(GamePieceView*)piece Position:(CGPoint)pt
{
	isPieceMoveing = YES;
}

////////////////////////////////////////////////////////
//停止
////////////////////////////////////////////////////////
-(void) gamePieceCAMoveEnd:(GamePieceView*)piece Position:(CGPoint)pt Finish:(BOOL)isFinish
{
	isPieceMoveing = NO;
}

////////////////////////////////////////////////////////
//移動先のタイル
////////////////////////////////////////////////////////
-(GameBoardTile*)gamePieceNextMovTile:(GamePieceView*)piece
{
	int				x1, y1, x2, y2;
	int				tileIndex, targetIndex;
	GameBoardTile*	ret_tile=nil;
	
	//移動先
	targetIndex = piece.targetTile.tileIndex;
	x2 = (targetIndex % TILE_W_COUNT);
	y2 = (targetIndex / TILE_W_COUNT);
	//現在の位置
	tileIndex = piece.tile.tileIndex;
	x1 = (tileIndex % TILE_W_COUNT);
	y1 = (tileIndex / TILE_W_COUNT);
	
	if(y2 == y1 && x2 == x1 )
		return	nil;
	
	double	xdis = fabs( x2 - x1 );
	double	ydis = fabs( y2 - y1 );
	
	//右上
	if( y2 <= y1 && x2 >= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
		}
	}
	//右下
	else if( y2 >= y1 && x2 >= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
		}
	}
	//左下
	else if( y2 >= y1 && x2 <= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex+TILE_W_COUNT];
		}
	}
	//左上
	else if( y2 <= y1 && x2 <= x1 ){
		//縦の距離が長い
		if( xdis <= ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-1];
		}
		//横の距離が長い
		else if( xdis > ydis ) {
			ret_tile = [gameView tileAtIndex:tileIndex-1];
			if( ret_tile.piece )
				ret_tile = [gameView tileAtIndex:tileIndex-TILE_W_COUNT];
		}
	}
	
	if( ret_tile.piece )
		return	nil;
	
	return	ret_tile;
}

////////////////////////////////////////////////////////
//移動先のタイル
////////////////////////////////////////////////////////
-(GameBoardTile*)gamePieceTargetMovTile:(GamePieceView*)piece {
	GameBoardTile*	tile=nil;
	if([self isNearPlayerPiece:piece]==NO){
		tile = [pieceView tile];
	}
	return	tile;
}

////////////////////////////////////////////////////////
//移動先を終了するか？
////////////////////////////////////////////////////////
-(BOOL)gamePieceIsMoveFinish:(GamePieceView*)piece {
	return	[self isNearPlayerPiece:piece];
}



@end
