
#import <QuartzCore/QuartzCore.h>
#import <math.h>
#import "GamePieceView.h"
#import "GameBoardTile.h"
#import "GameController.h"


@implementation GamePieceView

@synthesize	delegate;
@synthesize	imageArray;
@synthesize	imageIndex;
@synthesize	speed;
@synthesize	type;		//（追加）
@synthesize	state;		//（追加）
@synthesize	isMoveing;
@synthesize	tile;
@synthesize	targetTile;
@synthesize	nextTile;

////////////////////////////////////////////////////////
//初期化
////////////////////////////////////////////////////////

-(id)initWithFrame:(CGRect)frame Images:(NSArray*)images Delegate:(id)del
{
	if (self = [super initWithFrame:frame]) {
		self.state = STATE_STAY;
		self.opaque = NO;
		imageIndex = 0;
		speed = 100;
		self.imageArray = images;
		self.delegate = del;
		[self update];
	}
	return self;
}



////////////////////////////////////////////////////////
//解放
////////////////////////////////////////////////////////
- (void)dealloc {
	[imageArray release];
	[tile release];
	[targetTile release];
	[super dealloc];
}


////////////////////////////////////////////////////////
//描画
////////////////////////////////////////////////////////
- (void)drawRect:(CGRect)rect {
	/*
	if( [imageArray count] ){
		
		int	offset=1, anime=0;
		//移動中
		if( isMoveing ){
			offset = 3;
			anime = animeIndex;
		}
		UIImage*	image = [imageArray objectAtIndex:(imageIndex*offset)+anime];
		[image drawInRect:rect blendMode:kCGBlendModeNormal alpha:1.0];
	}
	 */
}


#pragma mark -

//タイマー
-(void)timerStart
{
	[self timerStop];
	moveTimer = [NSTimer scheduledTimerWithTimeInterval:1.0 / 60 
												 target:self 
											   selector:@selector(moveAction:) 
											   userInfo:nil 
												repeats:YES];
}

-(void)timerStop
{
	[moveTimer invalidate];
	moveTimer = nil;
}

-(void) moveAction:(NSTimer*)timer
{
	BOOL	update = NO;
	
	delay_mov += 1.0 / 60;
	//移動中
	if( isMoveing ){
		if(delay_mov >= 0.1){
			update = YES;
			animeIndex++;
			if(animeIndex > 2)
				animeIndex = 0;
		}
	}
	//停止中
	else {
		animeIndex = 0;
		////////////////////////////////////////////////////
		//オオカミの場合... （追加）
		if(type==TYPE_WLOF) {
			//デリゲートに移動先のタイルを要求する
			if( [delegate respondsToSelector:@selector(gamePieceTargetMovTile:)] ){
				GameBoardTile*		ret_tile = [delegate gamePieceTargetMovTile:self];
				if( ret_tile ){
					[self moveWith:ret_tile];
				}
			}
		}
		////////////////////////////////////////////////////
	}
	
	if(update)
		//[self setNeedsDisplay];
		[self update];
	
	if(delay_mov > 0.1)
		delay_mov = 0;
}



#pragma mark -

////////////////////////////////////////////////////////
//移動タイル設定
////////////////////////////////////////////////////////
-(void)moveWith:(GameBoardTile*)movTile
{
	//停止中
	if( isMoveing==NO ){
		self.targetTile = movTile;
		//デリゲートに移動先のタイルを要求する
		if(tile && [delegate respondsToSelector:@selector(gamePieceNextMovTile:)]){
			GameBoardTile*	mvTile = [delegate gamePieceNextMovTile:self];
			if( mvTile ){
				self.nextTile = mvTile;
				mvTile.piece = self;
				[self movePosition:[self.nextTile center]];
			}
		}
	}
	//移動中
	else {
		//移動先が違うので再設定
		if(self.targetTile != movTile){
			self.targetTile = movTile;
		}
	}
}


////////////////////////////////////////////////////////
//位置移動
////////////////////////////////////////////////////////
-(void)movePosition:(CGPoint)toPoint
{
	moveLocation = toPoint;
	//位置移動のためのアニメーションクラス
	CABasicAnimation*	anime = [CABasicAnimation animationWithKeyPath:@"position"];
	CGPoint	fromPt = self.layer.position;
	
	//移動２点間の距離を求め移動時間を求める
	double	x = fabs(toPoint.x-fromPt.x);
	double	y = fabs(toPoint.y-fromPt.y);
	double	moveLen = sqrt((x*x)+(y*y));
	anime.duration = moveLen / speed;
	
	//向き
	if(fromPt.y > toPoint.y){
		if(fromPt.x > toPoint.x){
			if(x > (y/2))
				imageIndex = MOVANGL_LEFT;
			else
				imageIndex = MOVANGL_BACK;
		}
		else{
			if(x > (y/2))
				imageIndex = MOVANGL_RIGHT;
			else
				imageIndex = MOVANGL_BACK;
		}
	}
	else{
		if(fromPt.x > toPoint.x){
			if(x > (y/2))
				imageIndex = MOVANGL_LEFT;
			else
				imageIndex = MOVANGL_FRONT;
		}
		else{
			if(x > (y/2))
				imageIndex = MOVANGL_RIGHT;
			else
				imageIndex = MOVANGL_FRONT;
		}
	}
	
	//移動の始点
	anime.fromValue = [NSValue valueWithCGPoint:fromPt];
	//移動の終点
	anime.toValue = [NSValue valueWithCGPoint:toPoint];		
	
	//アニメーションをレイヤーに設定
	isMoveing = YES;
	self.state = STATE_MOVEING;
	anime.delegate = self;
	[self.layer addAnimation:anime forKey:@"animatePosition"];
	
	//移動先に移動させる
	CGRect			aFrame = [self frame];
	CGPoint			nowPt = toPoint;
	[self.layer setFrame:CGRectMake(nowPt.x-(aFrame.size.width/2), nowPt.y-(aFrame.size.height/2), aFrame.size.width, aFrame.size.height)];
}

-(void)moveCancelPosition
{
	[self.layer removeAnimationForKey:@"animatePosition"];
	CALayer*	layer = [[self layer] presentationLayer];
	CGPoint		pt = layer.position;
	self.center = pt;
}


-(void)update
{
	if( [imageArray count] ){
		
		int	offset=1, anime=0;
		//移動中
		if( isMoveing ){
			offset = 3;
			anime = animeIndex;
		}
		UIImage*	image = [imageArray objectAtIndex:(imageIndex*offset)+anime];
		self.layer.contents = (id)image.CGImage;
	}
}

-(void)moveingFinish {
	
	isMoveing = NO;
	self.state = STATE_STAY;
	imageIndex = MOVANGL_FRONT;
	animeIndex = 0;
	//デリゲートメソッド
	if( [delegate respondsToSelector:@selector(gamePieceCAMoveEnd:Position:Finish:)]){
		CALayer*	layer = [[self layer] presentationLayer];
		CGPoint		pt = layer.position;
		[delegate gamePieceCAMoveEnd:self Position:pt Finish:YES];
	}
	[self update];
}

#pragma mark -
#pragma mark -- CAAnimationDelegate --

//アニメーション開始
- (void)animationDidStart:(CAAnimation *)theAnimation
{
	//デリゲートメソッド
	if( [delegate respondsToSelector:@selector(gamePieceCAMoveStart:Position:)]){
		CALayer*	layer = [[self layer] presentationLayer];
		CGPoint		pt = layer.position;
		[delegate gamePieceCAMoveStart:self Position:pt];
	}
}

//アニメーション停止
- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag
{
	//現在の位置を更新
	self.tile.piece = nil;
	self.tile = nextTile;
	self.tile.piece = self;
	
	//移動先に到達
	if(targetTile==nextTile){
		[self moveingFinish];
	}
	//まだ到達していない
	else {
		if(self.type==TYPE_WLOF) {
			NSLog(@"WLOF ---> %d", self.tile.tileIndex);
		}
		BOOL	isContinue=NO;
		//デリゲートに移動を終了するか確認する
		if( [delegate respondsToSelector:@selector(gamePieceIsMoveFinish:)]){
			if( [delegate gamePieceIsMoveFinish:self] ){
				[self moveingFinish];
			}
			else {
				isContinue = YES;
			}
		}
		//デリゲートに移動先のタイルを要求する
		if(isContinue && tile && [delegate respondsToSelector:@selector(gamePieceNextMovTile:)]){
			GameBoardTile*	ret_tile = [delegate gamePieceNextMovTile:self];
			if(ret_tile){
				self.nextTile = ret_tile;
				[self movePosition:[self.nextTile center]];
			}else{
				[self moveingFinish];
			}
		}
	}

}




@end
